#!/usr/bin/perl -w
# Generate an /etc/network/interfaces script based on the ip a command

use strict;
use Getopt::Std;
use Data::Dumper;
use Net::Netmask;
use JSON;
#iface only => dhcp mode
#address and mask => static mode
#no option => no changes
my %options;
getopts('i:a:m:g:',\%options);

my %iface = ();  # name -> iface info hash
my $ciface;  # current iface name

# First, read interfaces from ifconfig
#
open (IFC,"/usr/sbin/ip -j a | ") || die ("Could not execute ip: $!\n");
my $output ="";
while(my $line = <IFC>)
{
    chomp $line;
    $output .=$line;
}
my $interface = decode_json($output);
foreach my $item( @$interface )
{
	$ciface = $item->{ifname};
	$iface{$ciface} = { };
	if($item->{link_type} eq 'ether')
	{
		$iface{$ciface}->{"type"} = "ethernet";
	}
	else
	{
		$iface{$ciface}->{"type"} = $item->{link_type};
	}
	foreach my $addr(@{$item->{addr_info}})
	{
		if($addr->{family} eq "inet")
		{
			$iface{$ciface}->{"ipv4"} = "yes";
			my $cidr = $addr->{local}.'/'. $addr->{prefixlen};
			my $block = Net::Netmask->new2( $cidr );
			$iface{$ciface}->{"ipv4_addr"} =  $addr->{local};
			$iface{$ciface}->{"ipv4_mask"} = $block->mask;
			$iface{$ciface}->{"ipv4_bcast"} = $addr->{broadcast}
		}
	}
}
close IFC;

# Now, read route information from netstat
#
open (ROU,"route -n | ") || die ("Could not execute route: $!\n");
while(my $line = <ROU>) {
	chomp $line;
	if ( $line =~ m/^([\d\.]+)\s+([\d\.]+)\s+([\d\.]+)\s+(\S+)\s+(\d+).*?\s+(\S+)$/) {
		my $dest = $1; my $gw = $2; my $mask = $3; my $flags = $4;
		my $metric = $5; my $if = $6;
		if ( defined ( $iface{$if} ) ) {
			if ( $dest eq "0.0.0.0" && $mask eq "0.0.0.0" ) {
# Default gateway
				$iface{$if}->{"gateway"} = $gw;
			} elsif ( $gw ne "0.0.0.0" )  {
# Specific (static) route
				push @{$iface{$if}->{"up"}} , "route add -net $dest netmask $mask gw $gw dev $if";
				push @{$iface{$if}->{"down"}} , "route del -net $dest netmask $mask gw $gw dev $if";
			}
		}
	}

}

close ROU;

foreach my $if (keys %iface) {
	if ($iface{$if}->{"type"} =~ m/loopback/i) {
		if ($iface{$if}->{"ipv4"} eq "yes") {
			print "auto lo\n";
			print "iface $if inet loopback\n";
		}
	}

	if ($iface{$if}->{"type"} =~ m/ethernet/i) {
		if ($iface{$if}->{"ipv4"}) {
			print "auto $if\n";
			print "allow-hotplug $if\n";

			if( defined $options{i} && $options{i} =~ m/$if/ && !defined $options{a} && !defined $options{g} ){##DHCP mode
				print "iface $if inet dhcp\n";
			}
			else{  #default or static mode
				print "iface $if inet static\n";
				if( defined $options{i} && $options{i} =~ m/$if/ ) { ##Interface qu'il faut configurer
					$iface{$if}->{"ipv4_addr"} = $options{a} if defined $options{a};
					$iface{$if}->{"ipv4_mask"} = $options{m} if defined $options{m};
				}

				if (defined $iface{$if}->{"ipv4_addr"}) {
					print "    address " . $iface{$if}->{"ipv4_addr"} . "\n";
				}
				if (defined $iface{$if}->{"ipv4_mask"}) {
					print "    netmask " . $iface{$if}->{"ipv4_mask"} . "\n";
				}

				if ( defined $options{g} ){
					if( $options{i} =~ m/$if/ ) {
						print "    gateway " . $options{g} . "\n";
					}
				}
				elsif (defined $iface{$if}->{"gateway"}) {
					print "    gateway " . $iface{$if}->{"gateway"} . "\n";
				}

				unless ( defined $options{i} && $options{i} =~ m/$if/ ){ #Script run sans options || Interface non spécifiée dans la ligne de commande
					if (defined $iface{$if}->{"ipv4_addr"}) {
						print "    broadcast " . $iface{$if}->{"ipv4_bcast"} . "\n";
					}
					if (defined $iface{$if}->{"pre-up"}) {
						while ( my $upcmd = pop @{$iface{$if}->{"pre-up"}} ) {
							print "    pre-up " . $upcmd . "\n";
						}
					}
					if (defined $iface{$if}->{"up"}) {
						while ( my $upcmd = pop @{$iface{$if}->{"up"}} ) {
							print "    up " . $upcmd . "\n";
						}
					}
					if (defined $iface{$if}->{"down"}) {
						while ( my $downcmd = pop @{$iface{$if}->{"down"}} ) {
							print "    down " . $downcmd . "\n";
						}
					}
					if (defined $iface{$if}->{"post-down"}) {
						while ( my $downcmd = pop @{$iface{$if}->{"pre-down"}} ) {
							print "    pre-down " . $downcmd . "\n";
						}
					}
				}

			}
		}
	}
	print "\n";
}
