#!/usr/bin/php
<?php
define('PATH_ESIA_CONFIGURE','/usr/sbin/esia-configure');
define('PATH_IP_COMMAND','/usr/sbin/ip');
define('PATH_DHCLIENT','/usr/sbin/dhclient');
define('NAME',"netconfigure.php");
function print_usage(){
	echo 'USAGE : '.NAME." -f <file to network conf> -t mgmt\n";
}

function print_help()
{
	print_usage();
	echo "\t".NAME." : configure le réseau d'une unity\n";
	echo "\t\t[-f] 	| --file		file to network conf\n";
	echo "\t\t[-t] 	| --type	 spécifie le type de configuration [mgmt|other] (default:mgmt)\n";
	echo "\t\t[-n] 	| --name	 si le type=other indique le nom de l'interface\n";
}

//Définition des paramètres d'exécution
$longopts  = array("file:","type:","name:","debug","help","enable","disable",'ip:');
$options = getopt("f:t:n:hDedi:",$longopts);

$debug = getParam($options, 'D', 'debug');
$filepath = getParam($options, 'f', 'file',NULL);
$type = getParam($options, 't', 'type',"mgmt");
$name = getParam($options, 'n', 'name',"");
$enable = getParam($options, 'e', 'enable');
$disable = getParam($options, 'd', 'disable');
$ip_address = getParam($options, 'i', 'ip');
Trace($debug,"Debug Enable");
if(empty($filepath))
{
	echo "Configuration file not found: $filepath\n";
	exit(1);
}
Trace($debug,"Open configuration file: $filepath");
$out = file_get_contents($filepath);
$json = json_decode($out,TRUE);

Trace($debug,"SUCCESS Open configuration file: $filepath");

if($type=="mgmt")
{
	Trace($debug,"get management interface");
	$mgmt = $json['mgmt'];
	Trace($debug,$mgmt);
	$param = '';
	if($mgmt['dhcp'] === false)
	{
		$dns="";
		foreach($mgmt['dns'] as $srv)
		{
			$dns.="$srv,";
		}
		if(!empty($dns))
		{
			$dns=rtrim($dns,',');
			$dns="-n $dns";
		}
		$param = "-S net  -M static -a ".$mgmt['ip_addr'].' -m '.$mgmt['netmask'].' -g '.$mgmt['gateway'].' -i '.$mgmt['name'].' '.$dns;
	}
	else
	{
		Trace($debug,"configure to DHCP");
		$param = "-S net -M dhcp -i ".$mgmt['name'];
	}
	Trace($debug,"Call esia configure: ");
	$execpath = PATH_ESIA_CONFIGURE.' '.$param;
	Trace($debug,$execpath);
	if(exec(PATH_ESIA_CONFIGURE.' '.$param.' -y', $output,$ret) == false)
	{
		echo "Exec Error\n";
	}
	foreach($output as $line)
	{
		print "$line\n";
	}
	exit($ret);
}
else
{
	if(empty($name))
	{
		echo "interface name is empty\n";
		exit(1);
	}
	if(isset($enable) && isset($disable))
	{
		echo "Error: specified enabled or disabled command\n";
		exit(1);
	}
	$index=-1;
	foreach($json as $key => $inter)
	{
		$ifname = $inter['name'];
		if($inter['vlan']!=0 && !empty($inter['vlan']))
		{
			$ifname = $inter['name'].'.'.$inter['vlan'];
		}
		if($ifname == $name)
		{
			$index = $key;
			break;
		}
	}
	if($index == -1)
	{
		echo "interface $name not found\n";
		exit(1);
	}
	if($index == 'mgmt')
	{
		echo "can't enable or disable mgmt interface\n";
		exit(1);

	}
	Trace($debug,"interface found $name ");
	try
	{
		if(isset($enable))
		{
			//on va activer l'interface en question
			$inter= $json[$index];
			Trace($debug,$inter);
			if($inter['vlan']!=0 && !empty($inter['vlan']))
			{
				Trace($debug,"Activation du vlan");
				Trace($debug,"enable main interface ".$inter['name']);
				execIPCommand($debug,'link set '.$inter['name'].' up');
				Trace($debug,"enable vlan interface $name vlan id ".$inter['vlan']);
				execIPCommand($debug,'link add link '.$inter['name'].' name '.$name.' type vlan id '.$inter['vlan']);
			}
			Trace($debug,"enable interface ".$name);
			execIPCommand($debug,'link set '.$name.' up');
			if($inter['dhcp']==true)
			{
				$tmp = execDHClient($debug,$name);
                $inter['ip_addr'] = $tmp['ip_addr'];
                $inter['netmask'] = $tmp['netmask'];
                $inter['gateway'] = $tmp['gateway'];
			}
			else
			{
				$ip = $inter['ip_addr'].'/'.mask2cidr($inter['netmask']);
				Trace($debug,"On fixe l'adresse IP ".$ip);
				execIPCommand($debug,'addr add '.$ip.' dev '.$name);
			}
			echo "Enable with success $name";
			//configure passerelle/arp
			enableSpecificRoute($debug,$inter,$ip_address);

		}
		elseif(isset($disable))
		{
			//clean configure passerelle/arp

			if($inter['dhcp']!==true)
			{
				$ip = $inter['ip_addr'].'/'.mask2cidr($inter['netmask']);
				Trace($debug,"On supprime l'adresse IP ".$ip);
				execIPCommand($debug,'addr del '.$ip.' dev '.$name);
			}
            else
            {
                disableDHClient($debug,$name);
            }
			if($inter['vlan']!=0 && !empty($inter['vlan']))
			{
				Trace($debug,"disable interface ".$name);
				execIPCommand($debug,'link set '.$name.' down');

				Trace($debug,"Désactivation du vlan");
				execIPCommand($debug,'link del '.$name);

			}
			Trace($debug,"disable interface ".$inter['name']);
			execIPCommand($debug,'link set '.$inter['name'].' down');
			echo "Disable with success $name";
		}
	}
	catch(Exception $e)
	{
		echo $e->getMessage();
		exit(1);
	}

}
function execIPCommand($debug,$params)
{
	$execpath= PATH_IP_COMMAND.' '.$params;
	Trace($debug,"EXEC $execpath");
	if(exec($execpath, $output,$ret) === false)
	{
		throw new Exception("Exec Error: ".$execpath);
	}
}
function execDHClient($debug,$name)
{
	$execpath= PATH_DHCLIENT.' '.$name;
	Trace($debug,"EXEC $execpath");
	if(exec($execpath, $output,$ret) === false)
	{
		throw new Exception("Exec Error: ".$execpath);
	}
	$execpath= PATH_IP_COMMAND.' --json addr show '.$name;
	Trace($debug,"EXEC $execpath");
	if(exec($execpath, $output,$ret) === false)
	{
		throw new Exception("Exec Error: ".$execpath);
	}
    $output = implode("\n",$output);
    $output = json_decode($output,TRUE);
    $output = $output[0];

    $ret = array();
    if( !isset($output['addr_info'][0]['local']) )
    {
        throw new Exception("No IPv4 address set on iface");
    }
    $ret['ip_addr']=$output['addr_info'][0]['local'];
    $ret['netmask']=$output['addr_info'][0]['prefixlen'];

    $leases = file_get_contents("/var/lib/dhcp/dhclient.leases");
    if(empty($leases))
    {
        throw new Exception("Unable to read lease file /var/lib/dhcp/dhclient.leases");
    }
    $leases = explode("\n",$leases);
    $iface = FALSE;
    $addr = FALSE;
    foreach($leases as $row) //Remarque: le fichier peut contenir plusieurs leases, la plus récente à la fin du fichier. Donc on prends cette dernière.
    {
        if( !$iface && strpos($row,'interface "'.$name.'"') !== FALSE) //On cherche la pemière occurence de l'interface.
        {
            $iface = TRUE;
        }
        if( $iface && strpos($row,'fixed-address') !== FALSE )
        {
            if( strpos($row,'fixed-address '. $ret['ip_addr']) !== FALSE )
            {
                $addr = TRUE;
            }
            else
            {
                $iface = FALSE;
            }
        }

        if( $addr && preg_match('/option routers ([\d\.]+);/',$row,$matches) )
        {
            $ret['gateway'] = $matches[1];
            $iface = FALSE;
            $addr = FALSE;
        }
    }

    if( !isset($ret['gateway']))
    {
        throw new Exception("No gateway found from DHCP leases files");
    }
    return $ret;
}

function disableDHClient($debug,$name)
{
	$execpath= PATH_DHCLIENT.' -r '.$name;
	Trace($debug,"EXEC $execpath");
	if(exec($execpath, $output,$ret) === false)
	{
		throw new Exception("Exec Error: ".$execpath);
	}
}

function enableSpecificRoute($debug,$inter,$ips)
{
	$ips = explode(',',$ips);
	$local_net_info = "";
	if (filter_var($inter['ip_addr'],  FILTER_VALIDATE_IP,FILTER_FLAG_IPV4))
	{
		$local_net_info = ipv4NetworkInformation($inter['ip_addr'], $inter['netmask']);
	}
	elseif (filter_var($inter['ip_addr'],  FILTER_VALIDATE_IP,FILTER_FLAG_IPV6))
	{
		$local_net_info = ipv6NetworkInformation($inter['ip_addr'], $inter['netmask']);
	}
    else
    {
        return;
    }
	foreach($ips as $ip)
	{
		$notSameNetwork = FALSE;
		$ip_version = 0;
		if (filter_var($ip,  FILTER_VALIDATE_IP,FILTER_FLAG_IPV4))
		{
			$ip_version = 4;
			Trace($debug,"VALID IPV4  $ip");
			$decip = ip2long($ip);
			$local_net = $local_net_info['network'];
			$local_br = $local_net_info['broadcast'];
			if($decip > $local_net && $decip < $local_br)
			{
				//on est dnas le même réseau local. il ne faut pas toucher au routage
				Trace($debug,"IP $ip is in same local network IPV4");
			}
			else
			{
				$notSameNetwork=TRUE;
			}
		}
		elseif (filter_var($ip,  FILTER_VALIDATE_IP,FILTER_FLAG_IPV6))
		{
			$ip_version = 6;
			Trace($debug,"VALID IPV6  $ip");
			$decip = ipv6ToBin($ip);
			$local_net = $local_net_info['network'];
			$local_br = $local_net_info['broadcast'];
			if($decip > $local_net && $decip < $local_br)
			{
				//on est dnas le même réseau local. il ne faut pas toucher au routage
				Trace($debug,"IP $ip is in same local network IPV6");
			}
			else
			{
				$notSameNetwork=TRUE;
			}
		}
		if($notSameNetwork==TRUE)
		{
			Trace($debug,"Modification du routage $ip");
			//modification ARP ou passerelle
			$remote_addr = $ip;
			$local_iface = $inter['name'];

			if($inter['vlan']!=0 && !empty($inter['vlan']))
			{
				$local_iface .= '.' . $inter['vlan'];
			}

			$local_gw = $inter['gateway'];
			if( $ip_version == 4 )
			{
				$cmd = "ip route add $remote_addr/32 dev $local_iface via $local_gw";
				echo "$cmd\n";
				if(exec($cmd, $output,$ret) === false)
				{
					throw new Exception("Exec Error: ".$cmd);
				}
			}
			elseif( $ip_version == 6 )
			{
				$cmd = "ip -6 route add $remote_addr/128 dev $local_iface via $local_gw";
				if(exec($cmd, $output,$ret) === false)
				{
					throw new Exception("Exec Error: ".$cmd);
				}
			}
			else
			{
				Trace($debug,"Unknown ip version $ip_version");
			}
		}

	}
}

function ipv4NetworkInformation ($ip_address, $netmask)
{
    if( is_numeric($netmask) ) //Cidr
    {
        $ip_nmask_long = 32 - $netmask;
        $ip_broadcast_invert = pow(2,$ip_nmask_long)-1;
        $ip_nmask_long = ip2long('255.255.255.255')-$ip_broadcast_invert;
    }
    else //netmask string
    {
        $ip_nmask_long = ip2long($netmask);
        $ip_broadcast_invert = ip2long('255.255.255.255')-$ip_nmask_long;
    }
    //convertion des IPv4 en décimal
    $ip_address_long = ip2long($ip_address);
    //On fait le & pour avoir l'ip du réseau en décimal
    $ip_net = $ip_address_long & $ip_nmask_long;
    //calcul de l'adresse de broadcast
    $ip_broadcast = $ip_address_long | $ip_broadcast_invert;
    return array('network'=>$ip_net,'broadcast'=> $ip_broadcast);
}
function ipv6NetworkInformation ($ip_address, $cidr)
{
    //convertion des IPv6 en binaire
    $binari6 = ipv6ToBin($ip_address);
	$hosts = (128 -  $cidr);
    $networks = 128 - $hosts;
	$_m = str_repeat('1', $networks).str_repeat('0', $hosts);
	$_hexMask = null;
	foreach(str_split($_m, 4) as $segment)
	{
    	$_hexMask .= base_convert($segment, 2, 16);
	}
	$netmask = substr(preg_replace('/([A-f0-9]{4})/', '$1:', $_hexMask), 0, -1);
	$binari_netmask = ipv6ToBin($netmask);


	$network = $binari6 & $binari_netmask;
    //calcul de l'adresse de broadcast
    $ip_broadcast_invert = ~$binari_netmask;
    $ip_broadcast = $network | $ip_broadcast_invert;

	return array('network'=>$network,'broadcast'=> $ip_broadcast);
}
function ipv6ToBin($ipaddress)
{
    $pton = @inet_pton($ipaddress);
    if (!$pton)
    {
    	return false;
    }
    $binari6 = current(unpack('a*',$pton));
	return  $binari6;
}
function mask2cidr($mask)
{
	if(is_numeric($mask))
	{
		return $mask;
	}
	$long = ip2long($mask);
	$base = ip2long('255.255.255.255');
  	return 32-log(($long ^ $base)+1,2);
}
function getParam($options, $shortName, $slongName, $default = null)
{
    $param = $default;
    if (isset($options[$shortName]))
    {
        $param = $options[$shortName];
    }
    elseif (isset($options[$slongName]))
    {
        $param = $options[$slongName];
    }
    // security for string
    if (is_string($param))
    {
        $param = strtolower(trim(str_replace(' ', '', $param)));
    }
    return $param;
}
function Trace($debug,$string)
{
	if(isset($debug))
	{
		if(is_array($string))
			print_r($string);
		else
			echo "$string\n";
	}
}
?>
